import QtQuick 1.1
import Box2D 1.0
import Expresso 1.0
import "sprites"
import "engine.js" as Engine

Item {
    id: root

    property int displacement: 0
    property alias fuel: player.fuel
    property alias fuelPlus: player.fuelPlus
    property alias running: foreground.running
    property alias verticalDisplacement: canvas.sceneY

    function init() {
        Engine.init();
        Engine.startGame(false);
    }

    function restart() {
        Engine.restartGame();
    }

    SpriteScene {
        id: canvas
        x: root.displacement
        width: root.width
        height: root.height

        automaticStep: false
        onAdvanced: Engine.tick();

        RawConnection {
            sender: foreground
            senderSignal: "stepped()"
            receiver: canvas
            receiverSlot: "tick()"
        }

        property real sceneX: 0
        property real sceneY: Math.min(0.0, player.y)
        property real sceneWidth: 10e8
        property real sceneHeight: canvas.height * 6.0

        World {
            id: foreground
            running: root.running
            x: canvas.sceneX
            y: -canvas.sceneY
            width: parent.width
            height: parent.height
            gravity: Qt.point(0, -20)

            HowToFlag {
                showSmallBird: false
                visible: screen.showHowTo
                x: parent.width * 0.4
                anchors.verticalCenter: parent.verticalCenter
            }

            Player {
                id: player
                z: 1
            }

            Floor {
                width: canvas.sceneWidth
                y: canvas.height - height
            }

            Ceil {
                width: canvas.sceneWidth
                y: -canvas.height - height
            }

            HighScore {
                x: screen.highScore * screen.metersByPixel
                y: 300
            }

            /*
            DebugDraw {
                world: foreground
                visible: true
                opacity: 0.75
                width: canvas.sceneWidth
                height: parent.height
            }
            */
        }

        MouseArea {
            anchors.fill: parent
            onPressed: player.fly();
            onReleased: player.stop();
        }
    }
}
